/* Copyright 2004, Jan D. Wolter, All Rights Reserved. */

/* Access/Update SQL Conference Database.
 *
 * This code is really just the first stub end of something that will probably
 * get more sophisticated in future releases as we move more information into
 * the relational database.
 */

#include "backtalk.h"

#ifdef CDB

#include "cdb_conf.h"
#include "sqlutil.h"
#include "sqlqry.h"
#include "entropy.h"
#include "vstr.h"


/* CDB_CREATE_CONF:  Create an entry for a new conference in the conference
 * table.  Various fields are passed in, and a unique integer conference ID
 * number is returned.  Returns 0 on error.
 *
 * For now, conference information still resides primarily in the conference
 * config file and such-like places, and this table just serves as a way to
 * relate the conference info in the file to the info in the CDB.  So we store
 * only the unique integer ID, which is used as a conference ID in the CDB
 * and the participation file name, which is pretty much the only unique
 * conference identifier available in the config file.
 */

int cdb_create_conf(char *pfile)
{
    SQLhandle sqlh;
    char *id;

    make_sql_connection();
    sqlh= qry_cdb_new_conf(pfile);

    id= sql_fetchstr(sqlh,0);
    sql_got(sqlh);

    return (id == NULL) ? 0 : atoi(id);
}

/* CDB_DEL_CONF:  Delete a conference table entry.
 */

void cdb_del_conf(char *pfile)
{
    make_sql_connection();
    qry_cdb_del_conf(pfile);
}


/* CDB_CONF_ID:  Given a participation file name, find the conference's
 * unique ID number.  If the conference doesn't have an table entry, create
 * one for it.  Return 0 if that creation fails.
 */

int cdb_conf_id(char *pfile)
{
    SQLhandle sqlh;
    char *id;
    int createit;

    make_sql_connection();
    sqlh= qry_cdb_conf_id(pfile);

    createit= (sql_nrows(sqlh) != 1 || (id= sql_fetchstr(sqlh,0)) == NULL);
    sql_got(sqlh);

    if (createit)
	return cdb_create_conf(pfile);
    else
	return atoi(id);
}


/* CDB_NEW_ITEM:  Create a new item table entry.  Return the unique ID of
 * the new item in malloced memory, or NULL if it fails.
 */

char *cdb_new_item(time_t startdate, time_t lastdate, int frozen, int retired,
    int nresp, char *login, uid_t uid, char *alias, char *title, char *keys,
    char *sum)
{
    SQLhandle sqlh;
    char *itemid;
    int i, n;

    make_sql_connection();

    /* It should be extremely rare for this loop to iterate more than once,
     * since the chances of a randomly generated itemid string already being
     * in use are vanishingly small.  There is a possibility of a race
     * condition between the check that the id is free and the insertion,
     * but that too is vanishingly unlikely.
     */
    for (i= 0; i < 10; i++)
    {
	itemid= get_noise();
	sqlh= qry_cdb_item_info(itemid);
	n= sql_nrows(sqlh);
	sql_got(sqlh);
	if (n == 0) break;
	free(itemid);
    }
    if (i >= 10) die("Holy Toledo Batman!  I couldn't find a free item ID!");

    qry_cdb_new_item(itemid, startdate, lastdate, frozen, retired, nresp,
			   login, uid, alias, title, keys, sum);

    return itemid;
}


/* CDB_ITEM_INFO:  Get various stuff from the item database, pointing the
 * given buffers to them.  The values pointed to are in temporary memory
 * with no promise of a long lifetime.  Any pointers passed in as NULL will
 * not be set.
 */

void cdb_item_info(char *itemid, char **authorid, uid_t *uid, char **alias, 
		   char **title, char **keys, char **sum)
{
    SQLhandle sqlh;

    make_sql_connection();

    sqlh= qry_cdb_item_info(itemid);
    if (authorid != NULL) *authorid= sql_fetchstr(sqlh,0);
    if (uid != NULL)
    {
	char *s= sql_fetchstr(sqlh,1);
	if (s == NULL) die("failed to get uid number from database");
	*uid= (uid_t)atoi(s);
    }
    if (alias != NULL) *alias= sql_fetchstr(sqlh,2);
    if (title != NULL) *title= sql_fetchstr(sqlh,3);
    if (keys != NULL) *keys= sql_fetchstr(sqlh,4);
    if (sum != NULL) *sum= sql_fetchstr(sqlh,5);
    sql_got(sqlh);
}


/* CDB_ITEM_TITLE:  Set a new title for a conference.
 */

void cdb_item_title(char *itemid, char *title)
{
    make_sql_connection();

    qry_cdb_item_title(itemid,title);
}


/* CDB_ITEM_SUM_KEY:  Set a new title for a conference.
 */

void cdb_item_sum_key(char *itemid, char *summary, char *keywords)
{
    make_sql_connection();

    qry_cdb_update_sum_key(itemid,summary,keywords);
}


/* CDB_NEW_LINK:  Create a new link table entry.
 */

void cdb_new_link(char *itemid, int confid, int itemno, time_t linkdate)
{
    make_sql_connection();
    qry_cdb_new_link(itemid, confid, itemno, linkdate);
}


/* CDB_UNLINK:  Unlink an item from a conference.  If this was it's only link,
 * Delete it from the item list.  Return true if not found.
 */

int cdb_unlink(int confid, int itemno)
{
    SQLhandle sqlh;
    char *s;
    char *itemid;
    int n;

    make_sql_connection();
    sqlh= qry_cdb_link_info(confid,itemno);
    if (sql_nrows(sqlh) != 1 || (itemid= sql_fetchstr(sqlh,0)) == NULL)
    {
	sql_got(sqlh);
	return 1;
    }
    sql_got(sqlh);

    qry_cdb_del_link(itemid,confid);

    sqlh= qry_cdb_n_links(itemid);
    if (sql_nrows(sqlh) != 1 || (s= sql_fetchstr(sqlh,0)) == NULL)
    {
	sql_got(sqlh);
	return 1;
    }
    sql_got(sqlh);
    n= atoi(s);

    if (n == 0) qry_cdb_del_item(itemid);

    return 0;
}


/* CDB_LINK_INFO:  Get unique item ID for the given item number within the
 * given conference.  If linkdate is non-null, return the link date there.
 * If the table entry doesn't exist, we return NULL.
 */

char *cdb_link_info(int confid, int itemno, time_t *linkdate)
{
    SQLhandle sqlh;
    char *itemid, *ld;

    make_sql_connection();
    sqlh= qry_cdb_link_info(confid,itemno);

    if (sql_nrows(sqlh) != 1 || (itemid= sql_fetchstr(sqlh,0)) == NULL)
    {
	sql_got(sqlh);
	return NULL;
    }

    if (linkdate != NULL)
    {
	char *ld= sql_fetchstr(sqlh,1);
	*linkdate= (time_t)atoi(ld);
    }
    sql_got(sqlh);
    return itemid;
}


/* CDB_N_LINKS:  Return the number of links to given item number.
 */

int cdb_n_links(char *itemid)
{
    SQLhandle sqlh;
    char *s;

    make_sql_connection();

    sqlh= qry_cdb_n_links(itemid);
    if (sql_nrows(sqlh) != 1 || (s= sql_fetchstr(sqlh,0)) == NULL)
    {
	sql_got(sqlh);
	return 0;
    }
    sql_got(sqlh);
    return atoi(s);
}


/* CDB_RELATED_ITEMS:  Get an item selector for up to "limit" related items
 * that match the given keywords string in the given conference.  If limit is
 * zero, find them all.  Don't pass a conference id of zero to this.  If
 * notitem is nonzero, that item will be deleted from the list.  The returned
 * string is in malloced memory and should be freed by the caller.
 */

char *cdb_related_items(char *keywords, int confid, int limit, int notitem)
{
    SQLhandle sqlh;
    VSTRING isel;
    char *s;
    int i, n= 0;

    vs_new(&isel, 20);

    if (*keywords != '\0')
    {
	make_sql_connection();
	sqlh= qry_cdb_related_items(keywords, confid, limit+1);
	while (n < limit && sql_nextrow(sqlh))
	{
	    /* Skip items with a score of zero */
	    s= sql_fetchstr(sqlh,0);
	    if (s == NULL || atoi(s) == 0)
		continue;

	    /* Skip excluded item */
	    s= sql_fetchstr(sqlh,2);
	    if (s == NULL || atoi(s) == notitem)
		continue;

	    if (vs_size(&isel) > 0) *vs_inc(&isel)= ',';
	    vs_cat(&isel,s);
	    n++;
	}
    }
    return vs_return(&isel);
}


/* CDB_NEW_REFERENCE:  Add a reference to an item.
 */

void cdb_new_reference(char *itemid, char *url, time_t date, char *source,
		       char *title, char *excerpt, char* blog)
{
    make_sql_connection();

    qry_cdb_new_reference(itemid,url,date,source,title,excerpt,blog);
}


/* CDB_N_REFERENCES:  Return the number of references to given item number.
 */

int cdb_n_references(char *itemid)
{
    SQLhandle sqlh;
    char *s;

    make_sql_connection();

    sqlh= qry_cdb_n_references(itemid);
    if (sql_nrows(sqlh) != 1 || (s= sql_fetchstr(sqlh,0)) == NULL)
    {
	sql_got(sqlh);
	return 0;
    }
    sql_got(sqlh);
    return atoi(s);
}


/* CDB_GET_REFERENCES:
 * Start a walk through the list of references for the given item.
 * Returns nothing.  Call cdb_next_reference() to get the first value.
 */

SQLhandle rsqlh= NULL;

void cdb_get_references(char *itemid)
{
    make_sql_connection();

    if (rsqlh != NULL)
	sql_got(rsqlh);

    rsqlh= qry_cdb_item_info(itemid);
}


/* CDB_NEXT_REFERENCE:
 * Return the first/next reference to the given item.  Return value is
 * 1 if we actually returned something, 0 otherwise.  Returned data are
 * in temporary memory without promise of a long lifetime.  Any pointers
 * passed in as NULL will not be set.
 */

int cdb_next_reference(char **url, time_t *date, char *type, 
		   char **title, char **excerpt, char **blog)
{
    if (rsqlh == NULL) return 0;

    if (!sql_nextrow(rsqlh))
    {
	sql_got(rsqlh);
	rsqlh= NULL;
	return 0;
    }

    if (url != NULL) *url= sql_fetchstr(rsqlh,0);
    if (date != NULL)
    {
	char *s= sql_fetchstr(rsqlh,1);
	if (s == NULL) die("failed to get date from database");
	*date= (uid_t)atol(s);
    }
    if (type != NULL)
    {
	char *s= sql_fetchstr(rsqlh,2);
	*type= (s == NULL) ? 'M' : s[0];
    }
    if (title != NULL) *title= sql_fetchstr(rsqlh,3);
    if (excerpt != NULL) *excerpt= sql_fetchstr(rsqlh,4);
    if (blog != NULL) *blog= sql_fetchstr(rsqlh,5);

    return 1;
}

#endif /* CDB */
